/*
    Robot servo controller

    Gets data from the client and sends the data to a connected microcontroller

    (C) 2004 Jason Hunt

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    Contact: huntjas2@msu.edu

    Compile with -lwsock32

    On windows 2000 and XP you must install a real mode port assess driver.
    I use "userport"

    FILE: control.cpp
*/


#include <windows.h>
#include <stdio.h>
#include "control.h"
#include "display.h"

char left_motor = 'S',
     right_motor = 'S';

unsigned char pan  = 128,
              tilt = 128;


/* sends a byte out to the parallel port */
void port_out(unsigned char val)
{
    asm(
        "push %%ax\n"
        "push %%dx\n"
        "mov $0x378, %%dx\n"
        "mov %0, %%al\n"
        "out %%al, %%dx\n"
        "pop %%dx\n"
        "pop %%ax\n"  : : "g" (val)
    );
}
/* end of port_out */


/* gets the status byte of the lpt port */
unsigned char port_in(void)
{
    unsigned char r;

    asm ("mov $0x379, %%dx\n"
         "in %%dx, %%ax\n"
         "mov %%al, %0\n" :"=r"(r) ::"%ax","%dx");

    return(r);
}
/* end of port_in */


/* return the number of seconds */
unsigned int get_hund( void )
{
    SYSTEMTIME t;

    GetSystemTime(&t);

    return(t.wMilliseconds / 10);
}
/* end of get_seconds */

/*******************************
*** End of Utility Functions ***
*******************************/




/*********************************************
*** Servo controller Interfacing Functions ***
*********************************************/


/* returns true if the micro is ready for data */
int ready(void)
{
    return(!(port_in() & 0x80));
}
/* end of ready */


/* wait for ready line from controller */
int wait_ready(bool state)
{
    unsigned int start =  get_hund();
    unsigned int cur = start;

  	while((ready() == state) && (cur - start < 25)) cur = get_hund();

    if (cur - start >= 25) return(1);

    return(0);
}
/* end of wait ready */



/* sends a byte of data to the servo bank */
int send_byte(unsigned char byte)
{
	char count = 9;  // loop will actually cycle eght times

	if (wait_ready(false)) return(1);			// wait for ready high

	while (--count)
	{
        if (byte & 0x01)
        {
            port_out(0x03); // data and clock
            if(wait_ready(true)) return(1);					// wait for ready low

            port_out(0x02); // just data
    		if(wait_ready(false)) return(1);			// wait for ready high
        }
        else
        {
            port_out(0x01); //just clock
            if(wait_ready(true)) return(1);					// wait for ready low

            port_out(0x00); // nothing
    		if(wait_ready(false)) return(1);			// wait for ready high
        }

		byte >>= 1; // shit next bit over
	}
    return(0);
}
/* end of send byte */


/* ends the position for the different control codes to the micro */
void send_motor_dir(char c)
{
	if (c == 'F') send_byte(50); else
	if (c == 'S') send_byte(128); else
	if (c == 'R') send_byte(210); 

}
/* end of send_motor_move */


/* sends all of the servo data to the micro */
void update_controller()
{
    if (debug) return;

    // try to send "bank update" command
    if (send_byte(0))
    {
        add_line("Unable to reset motor controller.");
        return;
    } 

    send_motor_dir(left_motor); // left motor
    send_motor_dir(right_motor); // right motor

    send_byte(pan); // pan
    send_byte(tilt); // tilt
}
/* end of update_servos */


/* home servos */
void reset_controller()
{
    add_line("Resetting controller.");

    right_motor = 'S';  // home positions
    left_motor  = 'S';
    pan = 128;
    tilt = 128;

    update_controller(); // reset servos
}
/* end of reset_servos */


/* set controller to a contol set */
void set_controller(char lm, char rm, unsigned char p, unsigned char t)
{
    left_motor = lm;    right_motor = rm;
    pan = p;    tilt = t;

    char out[1024];
    sprintf(out, "Left: %c Right %c Pan: %d Tilt: %d", lm, rm, p, t);
    add_line(out);

    if (debug)
        add_line("Debug mode: no update");
    else
        update_controller();
}
/* end of set controller */


/****************************************************
*** End of Servo controller Interfacing Functions ***
****************************************************/


