/*
    Robot servo controller

    Gets data from the client and sends the data to a connected microcontroller

    (C) 2004 Jason Hunt

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    Contact: huntjas2@msu.edu

    Compile with -lwsock32

    On windows 2000 and XP you must install a real mode port assess driver.
    I use "userport"

    FILE: network.cpp
*/


#include <windows.h>
#include <stdio.h>
#include "network.h"
#include "display.h"
#include "control.h"

#define rec_buffer_size 1024    //size of thje receive buffer

SOCKET listen_socket;
SOCKET client_socket;

int rec_data = 0;           // true if date received
int connected = 0;          // true if date connected


/************************
*** Network Functions ***
************************/

/* configure and init the network */
void startup_network(HWND hwnd)
{
    add_line("Starting network.");

    // fire up winsock
    WSADATA wsda;
    WSAStartup(MAKEWORD(1, 1),&wsda);

    // create listen socket
    listen_socket = socket(AF_INET,SOCK_STREAM,IPPROTO_TCP);

    if (listen_socket == INVALID_SOCKET)
    {
        add_line("FATAL: Socket error.");
        return;
    }

    u_long arg = 1; // arg to allow non blocking

    // make accept non bocking
    if (ioctlsocket(listen_socket, FIONBIO, &arg) != 0)
    {
        add_line("FATAL: Unable to make listen socket non blocking.");
        return;
    }
    // setup parameters
    SOCKADDR_IN server;
    server.sin_addr.s_addr = INADDR_ANY;
    server.sin_port = htons(PORT);
    server.sin_family = AF_INET;

    // bind socket
    if(bind(listen_socket, (LPSOCKADDR) &server, sizeof(struct sockaddr)) == SOCKET_ERROR)
    {
        add_line("FATAL: Unable to bind.");
        return;
    }

    // setup listen
    if(listen(listen_socket,10) == SOCKET_ERROR)
    {
        add_line("FATAL: Error Listening.");
        return;
    }

    WSAAsyncSelect(listen_socket, hwnd, WM_NETEVENT, FD_WRITE | FD_CONNECT |
                                            FD_READ | FD_CLOSE);

    WSAAsyncSelect(listen_socket, hwnd, WM_NETEVENT, FD_READ | FD_WRITE |
                                            FD_ACCEPT | FD_CLOSE);

    add_line("Startup complete.");
}
/* end of startup network */


/* called when a client is trying to connect */
void network_accept(void)
{
    sockaddr_in client; // the client socket

    int client_size = sizeof(client);

    client_socket = accept(listen_socket,(struct sockaddr*)&client, &client_size);

    u_long arg = 1;  // make client non blocking

    if (ioctlsocket(client_socket, FIONBIO, &arg) != 0)
    {
        add_line("Unable to make client socket non blocking.");
        return;
    }

    if (client_socket == INVALID_SOCKET)
    {
        add_line("Accept error.");
        return;
    }

    // display client address

    char out[1024];

    sprintf(out, "Connection from: %s",inet_ntoa(client.sin_addr));
    add_line(out);

    connected = 1;
}
/* end of network accept */


/* does whatever needs doing based on the control packet from the client */
void apply_command(char *comm)
{
    if (comm[0] == 0x01 && (!space_key)) // update movement data
    {
        rec_data = 1;

        // set controller position
        set_controller(comm[1], comm[2], comm[3], comm[4]);

        if (send(client_socket, comm, 5, 0) == SOCKET_ERROR)
            add_line("Unable to send back to client.");

    } else
    if (comm[0] == 0)
        rec_data = 1; // keep alive
}
/* end of apply_command */


/* called when client has data for us */
void read_client_data(void)
{
    char buffer[rec_buffer_size]; // receive buffer
    char packet[5];

    int n_bytes = recv(client_socket, buffer, rec_buffer_size, 0);

    // read the entire buffer
    for (int n = 0; n < n_bytes / 5; n++)
    {
        memcpy(packet, buffer + n * 5, 5);

        apply_command(packet);
    }

    // buffer empty
    if (n_bytes == 0)
    {
        add_line("No data.");
    }
}
/* end of read_client_data */


/* lost connection, make robot safe */
void reset_network(void)
{
    add_line("No client data. Autoreset.");

    reset_controller();

    connected = 0;

    closesocket(client_socket);
}
/* end of reset network */


/* called when the client disconnets cleanly */
void  disconnect_client(void)
{
    closesocket(client_socket);     // close client socket
    connected = 0;
    reset_network();
}
/* end of disconnect client */


/* shutdown win sock */
void shutdown_network(void)
{
    disconnect_client();

    closesocket(client_socket);
    closesocket(listen_socket);     // close server socket

    WSACleanup();
}
/* end of shutdown_network */


/* deal with network messages */
void network_message(LPARAM lParam)
{
    switch (WSAGETSELECTEVENT(lParam))
    {
        case FD_ACCEPT:
            network_accept();
            break;

        case FD_CONNECT:
            break;

        case FD_CLOSE:
            add_line("Client disconnect");
            disconnect_client();
            break;

        case FD_READ:
            read_client_data();
            break;

        case FD_WRITE:
            break;
    }
}
/* end of network_message */


/*******************************
*** End of Network Functions ***
*******************************/


